
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <GL/glew.h> // http://glew.sourceforge.net/basic.html

#include <GL/glfw.h> // http://www.glfw.org/GLFWReference27.pdf

// http://www.khronos.org/files/opengl-quick-reference-card.pdf
#include "shaders.h"
#include "geometry.h"
#include "textures.h"
#include "fbo.h"

#ifndef M_PI
#define M_PI 3.141592654
#endif

GLuint winWidth=768;
GLuint winHeight=768;

float angleX=0.0;
float angleY=0.0;

int running=1;

//-------------- textures --------------

// Texture IDs
GLuint gColorMapID=0;
GLuint gGlossMapID=0;
GLuint gNormalMapID=0;

// ------------- geometry --------------
SModel modelHead;

// ------------- shaders --------------
GLuint shVertPosID=0;
GLuint shTexCoordID=0;

GLuint shVertNormalID=0;
GLuint shTangentID=0;
GLuint shBinormalID=0;

GLuint shAngleID=0;
GLuint shLightPosID=0;

float gAngleVal=0.0;

GLhandleARB gGLSLprog=0;

class CFrameBuffer
{
public:
    GLuint w,h;
    GLuint handle;

    GLuint depthRB;
    GLuint colorTex0;
};

GLuint createFBO(CFrameBuffer& FB,GLuint w,GLuint h)
{
    FB.w=w;
    FB.h=h;
	// create FrameBufferObject
	
	// bind frame buffer

	// create data storage for buffers (color buffers, depth buffer)

	// attach data storages to FBO 

	checkFBO(); // check FBO for errors

	// disable frame buffer
	return FB.handle;
}



void onInit()
{
// load texture
	gColorMapID=loadTGA("./textures/Walker_Head_DF.tga");
	gGlossMapID=loadTGA("./textures/Walker_Head_GLOSS.tga");
	gNormalMapID=loadTGA("./textures/Walker_Head_BP_normal.tga");

// load shaders
    gGLSLprog=createProgram("shaders/minimal.vert","shaders/minimal.frag");

	GLuint shColorMapID=glGetUniformLocation(gGLSLprog,"colorMap");
	GLuint shGlossMapID=glGetUniformLocation(gGLSLprog,"glossMap");
	GLuint shNormalMapID=glGetUniformLocation(gGLSLprog,"normalMap");

    glUseProgramObjectARB(gGLSLprog);

    glUniform1i(shColorMapID,0);
    glUniform1i(shGlossMapID,1);
    glUniform1i(shNormalMapID,2);

    glUseProgramObjectARB(0);

	shVertPosID=glGetAttribLocation(gGLSLprog,"inVertPos");
	shTexCoordID=glGetAttribLocation(gGLSLprog,"inTexCoord");
	shVertNormalID=glGetAttribLocation(gGLSLprog,"inVertNormal");

	shTangentID=glGetAttribLocation(gGLSLprog,"inTangent");
	shBinormalID=glGetAttribLocation(gGLSLprog,"inBinormal");

	shLightPosID=glGetUniformLocation(gGLSLprog,"inLightPos");
// load geometry
	loadGeometry(&modelHead);

	glGenBuffers(1, &modelHead.vPosVB); // positions
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.vPosVB);
    glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*3*sizeof(float), modelHead.vPosArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.vPosArr;

	glGenBuffers(1, &modelHead.vNormalVB); // vertnormals
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.vNormalVB);
	glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*3*sizeof(float), modelHead.vNormalArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.vNormalArr;

	glGenBuffers(1, &modelHead.vTangentVB);
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.vTangentVB);
	glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*3*sizeof(float), modelHead.vTangentArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.vTangentArr;

	glGenBuffers(1, &modelHead.vBinormalVB);
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.vBinormalVB);
	glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*3*sizeof(float), modelHead.vBinormalArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.vBinormalArr;

	glGenBuffers(1, &modelHead.texCoordVB); // texcoords
	glBindBuffer(GL_ARRAY_BUFFER, modelHead.texCoordVB);
	glBufferData(GL_ARRAY_BUFFER, modelHead.vertCnt*2*sizeof(float), modelHead.texCoordArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.texCoordArr;

	glGenBuffers(1, &modelHead.triangleIB); // Index Buffer
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,modelHead.triangleIB);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, modelHead.indexCnt*sizeof(unsigned int), modelHead.indexArr,GL_STATIC_DRAW_ARB);
	delete [] modelHead.indexArr;
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
}

void renderTextureSquare(GLuint texID)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, texID);
	glEnable(GL_TEXTURE_2D);

	glColor4f(1.0,1.0,1.0,1.0);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);
		glVertex2f(-1.0, -1.0);
		glTexCoord2f(1.0, 0.0);
		glVertex2f(1.0, -1.0);
		glTexCoord2f(1.0, 1.0);
		glVertex2f(1.0, 1.0);
		glTexCoord2f(0.0, 1.0);
		glVertex2f(-1.0, 1.0);
	glEnd();

	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
}

/* handle rendering */
void onRender()
{

	glDisable(GL_CULL_FACE);
    glEnable(GL_DEPTH_TEST);

	glClearColor(0.4,0.1,0.1,1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gColorMapID);
    glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, gGlossMapID);
    glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, gNormalMapID);
    glEnable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    glTranslatef( 0.0, 0.0, -2.0 );

    glRotatef(angleY,0,1,0);
	glRotatef(angleX,1,0,0);

// setup shader
    glUseProgramObjectARB(gGLSLprog);
// set light position
    glUniform3f(shLightPosID,5.0*sin(gAngleVal),0.0,5.0*cos(gAngleVal)-3.0);
	gAngleVal+=0.1;
// bind VBO - positions
	glBindBuffer( GL_ARRAY_BUFFER, modelHead.vPosVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shVertPosID,3,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shVertPosID);
// !!! OpenGL < 3.0 requires GL_VERTEX_ARRAY enabled to render geometry
	glVertexPointer(3,GL_FLOAT,0,0);
	glEnableClientState(GL_VERTEX_ARRAY);

// bind VBO - texcoords
	glBindBufferARB( GL_ARRAY_BUFFER_ARB, modelHead.texCoordVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shTexCoordID,2,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shTexCoordID);

	glBindBufferARB( GL_ARRAY_BUFFER_ARB, modelHead.vNormalVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shVertNormalID,3,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shVertNormalID);

	glBindBufferARB( GL_ARRAY_BUFFER_ARB, modelHead.vTangentVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shTangentID,3,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shTangentID);

	glBindBufferARB( GL_ARRAY_BUFFER_ARB, modelHead.vBinormalVB);	// bind ARRAY BUFFER with glBuffer of vertex positions
	glVertexAttribPointerARB(shBinormalID,3,GL_FLOAT,GL_FALSE,0,0);
	glEnableVertexAttribArrayARB(shBinormalID);

// VBOs done
	glBindBufferARB( GL_ARRAY_BUFFER_ARB, 0);	// unbind ARRAY BUFFER

// bind IB
	glBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, modelHead.triangleIB);	// bind ELEMENT ARRAY BUFFER with glBuffer of indices
	glDrawElements(GL_TRIANGLES,modelHead.indexCnt,GL_UNSIGNED_INT,0);
	glBindBufferARB( GL_ELEMENT_ARRAY_BUFFER_ARB, 0);	// unbind ELEMENT ARRAY BUFFER

// clean-up
	glDisableVertexAttribArray(shVertPosID);
	glDisableVertexAttribArray(shTexCoordID);
	glDisableVertexAttribArray(shVertNormalID);
	glDisableVertexAttribArray(shTangentID);
	glDisableVertexAttribArray(shBinormalID);

	glDisableClientState(GL_VERTEX_ARRAY);

	glUseProgramObjectARB(0);

	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE2);
	glDisable(GL_TEXTURE_2D);

}


/* new window size callback */
void GLFWCALL onReshape( int width, int height )
{
  GLfloat h = (GLfloat) height / (GLfloat) width;
  GLfloat xmax, znear, zfar;

  znear = 0.1f;
  zfar  = 30.0f;
  xmax  = znear*1.0f;

  glViewport( 0, 0, (GLint) width, (GLint) height );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glFrustum( -xmax, xmax, -xmax*h, xmax*h, znear, zfar );

  winWidth=width;
  winHeight=height;
}

/* handle keyboard input */
void GLFWCALL onKey( int k, int action )
{
  if( action != GLFW_PRESS ) return;

  switch (k) {
    case GLFW_KEY_ESC:
        running = 0;
    break;
    default:
        return;
  }
}

/* handle mouse move event */
void GLFWCALL onMouseMove( int x, int y )
{
    int button = GLFW_MOUSE_BUTTON_LEFT;
 /*
 GLFW_MOUSE_BUTTON_LEFT
 GLFW_MOUSE_BUTTON_RIGHT
 GLFW_MOUSE_BUTTON_MIDDLE
 */
    int state = glfwGetMouseButton( button );
    if (state == GLFW_PRESS)
    {
        fprintf(stderr,"%d %d\r",x,y);
    // process
    	angleY=x;
	    angleX=y;
    }
}

/* program entry */
int main(int argc, char *argv[])
{
    if( !glfwInit() )
    {
        fprintf( stderr, "Failed to initialize GLFW\n" );
        exit( EXIT_FAILURE );
    }
/*
glfwOpenWindow(int width, int height,
    int redbits, int greenbits, int bluebits, int alphabits,
    int depthbits, int stencilbits,
    int mode)
*/
    if( !glfwOpenWindow( winWidth,winHeight, 0,0,0,0, 16,0, GLFW_WINDOW ) )
    {
        fprintf( stderr, "Failed to open GLFW window\n" );
        glfwTerminate();
        exit( EXIT_FAILURE );
    }

    GLenum err = glewInit();
	if (err != GLEW_OK)
    {
		fprintf(stderr,"Error glew initialization failed (%s).\n",glewGetErrorString(err));
    }
    fprintf(stderr, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));

    glfwSetWindowTitle( "cviko_06" );
    glfwEnable( GLFW_KEY_REPEAT );
    glfwSwapInterval( 1 ); // VSync 1x

    // Set callback functions
    glfwSetWindowSizeCallback( onReshape );

    glfwSetKeyCallback( onKey );
    glfwSetMousePosCallback( onMouseMove );

    onInit();

    // Main loop
    while( running )
    {
         onRender();
        // Swap buffers
        glfwSwapBuffers();
        glfwSleep( 0.01 );    // don't eat whole CPU time
        // Was the window closed?
        if( !glfwGetWindowParam( GLFW_OPENED ) )
        {
            running = 0;
        }
    }

    // Terminate GLFW
    glfwTerminate();

    // Exit program
    exit( EXIT_SUCCESS );
}

